import pintora, { configApi, PintoraConfig, themeRegistry, GraphicsIR, Mark, DiagramArtistOptions } from '@pintora/core'
// import cloneDeep from 'clone-deep'

export function testDraw(code: string, extraOptions: Partial<DiagramArtistOptions> = {}) {
  let success = true
  const result = pintora.parseAndDraw(code, {
    ...extraOptions,
    onError(err) {
      console.error(err)
      success = false
    },
  })
  if (!success) {
    throw Error('testDraw fail')
  }
  return result
}

export function prepareDiagramConfig() {
  configApi.setConfig<PintoraConfig>({
    themeConfig: {
      theme: 'default',
      darkTheme: 'dark',
      themeVariables: themeRegistry.themes.default,
    },
  })
}

const GEOMETRY_ATTRS = ['x', 'y', 'width', 'height', 'x1', 'x2', 'y1', 'y2', 'path', 'points', 'margin']
const MARK_IGNORE_FIELDS = ['matrix']

/**
 * width/height and other text dimension related attributes may differ from test machine's default font.
 * ignore them for now to pass CI tests.
 */
export function stripGraphicIRForSnapshot(ir: GraphicsIR) {
  const cloned = ir
  delete cloned.height
  delete cloned.width

  const visited: Map<Mark, boolean> = new Map()

  function processMark(mark: Mark) {
    if (!mark) return
    if (visited.has(mark)) return

    visited.set(mark, true)
    const attrs = mark.attrs
    if (attrs) {
      GEOMETRY_ATTRS.forEach(k => {
        if (k in attrs) delete attrs[k]
      })
    }

    MARK_IGNORE_FIELDS.forEach(k => {
      if (k in mark) delete mark[k]
    })

    if ('children' in mark) {
      mark.children.forEach(child => processMark(child))
    }
  }

  processMark(cloned.mark)

  return cloned
}

export function stripDrawResultForSnapshot(result: ReturnType<typeof pintora.parseAndDraw>) {
  return stripGraphicIRForSnapshot(result.graphicIR)
}
